;    awnetwork.pbi
;    Version 0.2 [2010/08/26]
;    Copyright (C) 2011 Ronny Krueger
;
;    This file is part of AWPB-Tools.
;
;    AWPB-Tools is free software: you can redistribute it and/or modify
;    it under the terms of the GNU Lesser General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    AWPB-Tools is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU Lesser General Public License for more details.
;
;    You should have received a copy of the GNU Lesser General Public License
;    along with AWPB-Tools.  If not, see <http://www.gnu.org/licenses/>.

;{ constants
#____NW_BUF_SIZE  = 15000
#____NW_CRLF      = Chr(13) + Chr(10)
#____NW_CD_ERR    = 0
#____NW_CD_OK     = -1
;}
;{ Prototypes
Prototype.i ____NW_CallbackFunction(StatusText.s)
;}
;{ Structures
Structure ____NW_NetworkConn
  ConnID.i
  ResultString.s
  ResultCode.i
  DataString.s
  TimeOut.i
  Retries.i
  Buffer.i  
  Host.s
  Port.w
  User.s
  Pass.s
  CallbackFunc.____NW_CallbackFunction
EndStructure
;}
;{ private functions
Procedure.b ____NW_SendNetworkBin(*nwconn.____NW_NetworkConn, *buf, buflen.i)
  Protected maxlen.i, sentlen.i, timeout.i, orglen.i
  
  ; sendet binrdaten; dies geschieht in Blcken von maximal 50000 Byte
  
  orglen  = buflen
  timeout = *nwconn\TimeOut
  While (buflen > 0) And (timeout > 0)
    maxlen  = buflen
    If maxlen > 50000 : maxlen = 50000 : EndIf
    sentlen = SendNetworkData(*nwconn\ConnID, *buf, maxlen)
    If sentlen > 0
      buflen - sentlen
      *buf   + sentlen
    Else
      Delay(50) : timeout - 50
    EndIf
  Wend

  If buflen > 0 : ProcedureReturn #False : EndIf
  
;   If *nwconn\CallbackFunc
;     *nwconn\CallbackFunc("Sent: " + Str(orglen) + " Bytes")
;   EndIf
  
  ProcedureReturn #True
EndProcedure
Procedure.b ____NW_SendNetworkStr(*nwconn.____NW_NetworkConn, msg.s, crlf.b)
  Protected ok.b, l.i, crlfvalue.w
  
  ; sendet den bergebenen String
  ; im Unicode-Modus wird er dazu vorher in einen ASCII-String konvertiert
  ; anschlieend wird, falls crlf = #true, ein CRLF gesendet
  
  ok = #True
  l  = Len(msg)
  
  CompilerIf #PB_Compiler_Unicode
    Protected  a.i, b.i, c.i

    If l <= #____NW_BUF_SIZE
      PokeS(*nwconn\Buffer, msg, l, #PB_Ascii)
      ok = ____NW_SendNetworkBin(*nwconn, *nwconn\Buffer, l)
    Else
      c = Int(l / #____NW_BUF_SIZE)
      b = l % #____NW_BUF_SIZE
      
      For a = 0 To c - 1
        PokeS(*nwconn\Buffer, Mid(msg, (a * #____NW_BUF_SIZE) + 1, #____NW_BUF_SIZE), #PB_Ascii)
        ok = ____NW_SendNetworkBin(*nwconn, *nwconn\Buffer, #____NW_BUF_SIZE)
        If Not ok : Break : EndIf
      Next a
      
      If b > 0 And ok
        PokeS(*nwconn\Buffer, Mid(msg, (c * #____NW_BUF_SIZE) + 1, b), #PB_Ascii)
        ok = ____NW_SendNetworkBin(*nwconn, *nwconn\Buffer, b)
      EndIf
    EndIf
  CompilerElse
    ok = ____NW_SendNetworkBin(*nwconn, @msg, l)
  CompilerEndIf
  
  If crlf
    crlfvalue = $0a0d
    ____NW_SendNetworkBin(*nwconn, @crlfvalue, 2)
  EndIf
    
;  If *nwconn\CallbackFunc
;    *nwconn\CallbackFunc("Sent: >" + msg + "<")
;  EndIf
  
  Debug("Sent; >" + msg + "<")      ;- TODO: Remove

  ProcedureReturn ok
EndProcedure
Procedure.s ____NW_ReceiveNetworkStr(*nwconn.____NW_NetworkConn)
  Protected len.i, result.s
  
  result = ""
  Repeat    
    len = ReceiveNetworkData(*nwconn\ConnID, *nwconn\Buffer, #____NW_BUF_SIZE)
    If len
      result + PeekS(*nwconn\Buffer, len, #PB_Ascii)
    EndIf
  Until len < #____NW_BUF_SIZE
  
;   If *nwconn\CallbackFunc
;     *nwconn\CallbackFunc("Received " + Str(Len(result)) + " Bytes: " + result + "<")
;   EndIf
  
    Debug("Received: "+ Str(Len(result)) + " Bytes: " + result + "<")   ;- TODO remove
  ProcedureReturn result
EndProcedure 
Procedure.i ____NW_WaitNetworkClientEvent(*nwconn.____NW_NetworkConn)
  Protected Event.i, TimeOut.i
  
  TimeOut = *nwconn\TimeOut
  Repeat
    Event = NetworkClientEvent(*nwconn\ConnID)
    If Not event
      Delay(50)
      TimeOut - 50
    EndIf
  Until Event Or TimeOut <= 0
  
  ProcedureReturn Event
EndProcedure
Procedure.i ____NW_GetResultCode(*nwconn.____NW_NetworkConn)
  Protected code.i
  
  If LCase(Left(*nwconn\ResultString, 3)) = "+ok"
    code = #____NW_CD_OK
  ElseIf LCase(Left(*nwconn\ResultString, 4)) = "-err"
    code = #____NW_CD_ERR
  Else
    code = Val(Left(*nwconn\ResultString, 3))
  EndIf
  
  ProcedureReturn code
EndProcedure
Procedure.i ____NW_ReadResult(*nwconn.____NW_NetworkConn)
  Protected dt.s
  
  *nwconn\ResultString = ""
  *nwconn\DataString   = ""
  *nwconn\ResultCode   = -1
 
  If ____NW_WaitNetworkClientEvent(*nwconn) = #PB_NetworkEvent_Data
    *nwconn\ResultString = ____NW_ReceiveNetworkStr(*nwconn)
    *nwconn\ResultCode   = ____NW_GetResultCode(*nwconn)
    *nwconn\DataString   = Mid(*nwconn\ResultString, FindString(*nwconn\ResultString, Chr(10), 1) + 1)
    While NetworkClientEvent(*nwconn\ConnID) = #PB_NetworkEvent_Data
      dt = ____NW_ReceiveNetworkStr(*nwconn)
      If dt : *nwconn\DataString + dt : EndIf
    Wend
  EndIf
  
  ProcedureReturn *nwconn\ResultCode
EndProcedure
Procedure   ____NW_ReadData(*nwconn.____NW_NetworkConn)
  Protected dt.s, retry.i
  
;  retry = *nwconn\Retries
  While (Right(*nwconn\DataString, 5) <> #____NW_CRLF + "." + #____NW_CRLF) ; And (retry > 0)
    If NetworkClientEvent(*nwconn\ConnID) = #PB_NetworkEvent_Data
      dt = ____NW_ReceiveNetworkStr(*nwconn)
      If dt : *nwconn\DataString + dt : EndIf
    Else
      Delay(50)
;     retry - 1
    EndIf
  Wend
EndProcedure
Procedure.b ____NW_Execute(*nwconn.____NW_NetworkConn, cmd.s, retries.i = 1)
  Protected res.b
  
  Repeat
    res = ____NW_SendNetworkStr(*nwconn, cmd, #True)
    Delay(100)
    ____NW_ReadResult(*nwconn)
    If Not res
      Delay(100)
      retries - 1
    EndIf   
  Until res Or (retries <= 0)
  
  ProcedureReturn res
EndProcedure
Procedure   ____NW_SetCallbackFunc(*nwconn.____NW_NetworkConn, *CallbackFuncPointer = #Null)
  *nwconn\CallbackFunc = *CallbackFuncPointer
EndProcedure
;}

